' /////////////////////////////////////////////////////////////////////////////
{
 Propeller C3 Onboard SPI A/D Converter c3io_drv_ad.spin
 AUTHOR: Joe Hohertz
 LAST MODIFIED: 2011-03-14
 VERSION: 0.01
 LICENSE: MIT

 This is a part of "c3io" which is an attempt to encapsulate the
 various C3 demos into re-usable objects.

 This driver is intended to support the on board A/D converter. The A/D converter
 is a Microchip MCP3202, dual channel, successive approximation 12-bit A/D converter
 with a nominal sample rate of 100k samples per second. You can find the data sheet
 for the converter here:

 http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en010532
 http://ww1.microchip.com/downloads/en/DeviceDoc/21034e.pdf

 The MCP3202 is accessed via the SPI bus channel 4. Reading the A/D is simply a matter
 of sending the correct SPI request and then reading the data back. Referring to Table 5.1
 in the MCP3202 datasheet, reading the value of either channel requires a single SPI
 transaction! You simply send a few control bits that indicate single or differential
 mode and which channel and the device returns a 12-bit value [0..4095] that represents
 the voltage on the channel relative to the Vdd/Vref pin. The C3 uses 5V for Vdd/Vref
 thus 0V on a channel results in 0 being returned and 5V on the channel input results
 in 4095 being returned. The call is so simple a function isn't even required,
 a single line of code reads the A/D with a generic SPI call:

     SPI_Write_Read( 17, %0001_1C_1_0000000000000, $FFF )

     (where C=(0|1) refers to channel 0,1)

 Nevertheless, there are two functions provided to access the A/D converter:

 PUB AD_Init        - initializes the A/D converter.
 PUB AD_Read( ... ) - reads channel 0/1 in single ended mode 5V range.

 The action all occurs in the main() function, below the main() are a number of "utility"
 functions  included in each demo for reference. If you wish you can break them out into an
 external spin module. They are mostly for parsing and various random functions needed
 in each demo. The only important set of functions for the C3 are the "SPI" bus control
 functions, pay close attention to those since they control the selection of the various
 SPI devices on the bus.

}
' /////////////////////////////////////////////////////////////////////////////


OBJ

  ' current drivers used in this version of SPI interface
  io    : "c3io"

' /////////////////////////////////////////////////////////////////////////////
' ANALOG TO DIGITAL API ///////////////////////////////////////////////////////
' /////////////////////////////////////////////////////////////////////////////

PUB AD_Init

{{
This function initializes the MCP3202 A/D converter. Currently, does nothing
more than asserts the CS then de-asserts it.

PARMS:  None.

RETURNS: Nothing.

}}
  ' select and de-select the A/D device on the SPI bus just to wake it up and
  ' make sure we know the state of the A/D
  setLock
  clearLock

' end AD_Init

' /////////////////////////////////////////////////////////////////////////////

PUB AD_Read( channel ) | _data

{{
This function reads from the MCP3202 the sent AD channel (0,1) in single ended mode and returns a
12-bit integer [0..4095] representing the value.

PARMS:  channel - [0,1] which channel to read.

RETURNS: returns value of A/D converter [0..4095]

}}

  ' read channel 0, SPI transaction is 17 bits total, control bits indicate single ended mode, channel (0,1)
  setLock
  _data := io.SPI_Write_Read( 17, %0001_10_1_0000000000000 | channel << 14, $FFF )
  clearLock

  ' return A/D value
  return ( _data )

' end AD_Read

PRI setLock
  io.setLock
  io.SPI_Select_Channel(io#SPI_CHAN_AD)
PRI clearLock
  io.SPI_Select_Channel(io#SPI_CHAN_NULL)
  io.clearLock

DAT
{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
'//////////////////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////


