' /////////////////////////////////////////////////////////////////////////////
{
 Propeller C3 Onboard SRAM Driver c3io_drv_sram.spin

  NOTE: There is a VM implementation for the SRAM access as well. See
       c3io_demo_sram_cache_sterm.spin

 AUTHOR: Joe Hohertz
 AUTHOR: Andre' LaMothe
 LAST MODIFIED: 2011-03-14
 VERSION: 0.01
 LICENSE: MIT

 This is a part of "c3io" which is an attempt to encapsulate the
 various demos into re-usable objects.

 This driver is intended to support the 2 32KB SPI SRAM chips
 found on the C3 board. It builds on top of the SPI driver

 The SRAM are implemented with (2) Microchip 23K256 Serial SPI SRAMs.

 The 23K256 supports a number of commands,
 we have  only implemented a couple here in these API functions (single byte read/write,
 sequential byte read/write, write status register). The little demo API is shown below
 that implements these:

 PUB SRAM_Init ( ... ) - initializes the SRAMs and sets them to sequential mode.
 PUB SRAM_Write( ... ) - writes bytes of data from a buffer to either SRAM.
 PUB SRAM_Fill( ... )  - fills either SRAM with a constant value.
 PUB SRAM_Read( ... )  - reads data from either SRAM into a buffer.
 PUB SRAM64K_MREAD( ... ) - models the pair of 32K SRAMs as a single memory of 64K and reads a byte from it.
 PUB SRAM64K_MWRITE( ... ) - models the pair of 32K SRAMs as a single memory of 64K and writes a single byte to it.

 However, there are many others you can implement, find out more by reviewing the chip
 specs and data sheets found here:

   http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
   http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

 The SRAMs only have 4 commands as shown below here:

 Instruction Name/Value   Description

  READ  0000 0011         Read data from memory array beginning at selected address
  WRITE 0000 0010         Write data to memory array beginning at selected address
  RDSR  0000 0101         Read STATUS register
  WRSR  0000 0001         Write STATUS register

 However, there are 3 modes of operation which are controlled via the status register
 write; byte mode, page mode, and sequential mode -- read the data sheet to learn more.

 The SRAMs are accessed via the SPI bus and are located on SPI channels 1 and 2.
 The SRAMs use a simple SPI command protocal (outlined in the data sheet above)
 that allow single and multiple byte reads and writes.

}
' /////////////////////////////////////////////////////////////////////////////


'//////////////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ///////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

CON

  ' SRAM function helper constants
  SPI_MODE_BYTE       = %00_00000_1
  SPI_MODE_PAGE       = %10_00000_1
  SPI_MODE_SEQUENTIAL = %01_00000_1

VAR

  long temp1

OBJ

  ' current drivers used in this version of SPI interface
  io    : "c3io"
  'serial     : "FullDuplexserial_drv_014.spin"       ' the full duplex serial driver

  ' /////////////////////////////////////////////////////////////////////////////
' SRAM API ////////////////////////////////////////////////////////////////////
' /////////////////////////////////////////////////////////////////////////////

PUB SRAM_Init ( mode ) | _spi_word
{{
This function initializes both banks of the Microchip 23K256 SRAMs
serial SPI SRAMs'. The initialization simply sets the STATUS register
of each SRAM into "byte", "page" or "sequential" read/write mode to allow more
efficient access of the SRAMs. The SRAM API currently relies on "sequential"
mode, so if yoo want to use them, this function should be called with SPI_MODE_SEQUENTIAL

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

PARMS:  mode - %00_00000_1 = SPI_MODE_BYTE
               %10_00000_1 = SPI_MODE_PAGE
               %01_00000_1 = SPI_MODE_SEQUENTIAL

RETURNS: Nothing.

}}

  ' first set SRAM bank 0 to sequential access mode (see page 6 of data sheet)

  ' write status register command "sequential access mode" along with disable "HOLD" to setup SRAM for sequential reading/writing
  setLock(0)
  io.SPI_Write_Read( 16, %0000000_1 << 8 |  mode , $FF )
  clearLock

  ' same for SRAM bank 1
  setLock(1)
  io.SPI_Write_Read( 16, %0000000_1 << 8 |  mode , $FF )
  clearLock


' end SRAM_Init

' /////////////////////////////////////////////////////////////////////////////
PUB SRAM_Status ( bank ) | _spi_word
{{
This will query the status register of one of the banks of SRAM

PARAMS:   bank - the bank # to get status on

RETURNS:  the returned status
}}
  setLock(bank)
  temp1 := io.SPI_Write_Read( 16, %00000101_00000000, $FF )
  clearLock

  return(temp1)

' end SRAM_Init
' /////////////////////////////////////////////////////////////////////////////


PUB SRAM_Write( bank, addr, num_bytes, src_buffer_ptr) | _index, _data, _spi_word
{{
This function writes a number of bytes to either bank of the Microchip 23K256
serial SPI SRAMs' from a buffer. The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  bank           - [0,1] the SRAM bank to write to, there are two banks, each holds 32K bytes
        addr           - address in SRAM to write to [0...32767]
        num_bytes      - number of bytes to write from source buffer to SRAM
        src_buffer_ptr - pointer to the buffer of bytes to write to SRAM

RETURNS: number of bytes written

}}

  ' test if there is anything to write?
  if (num_bytes => 1)

    ' select SPI channel for requested SRAM bank
    setLock(bank)

    ' set instruction word ( write sequential byte:8 | address:16 )
    _spi_word := (%00000010 << 16) + (addr)

    ' and now send command to SRAM for sequential writing
    io.SPI_Write_Read( 24, _spi_word, $FF )

'    ' write the bytes into the SRAM
'    repeat _index from 0 to num_bytes-1
'      ' get byte to write
'      _data := byte[ src_buffer_ptr][ _index ]
'
'      ' write the byte
'      io.SPI_Write_Read( 8, _data, $FF )
    io.SPI_Write_Bytes( src_buffer_ptr, num_bytes )

    ' return bytes written
    clearLock
    return( num_bytes )

  else
    ' catch error
    return 0

' end SRAM_Write

' /////////////////////////////////////////////////////////////////////////////

PUB SRAM_Fill( bank, addr, num_bytes, value) | _index, _data, _spi_word
{{
This function writes a constant value to either bank of the Microchip 23K256
serial SPI SRAMs' from a buffer. The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  bank           - [0,1] the SRAM bank to write to, there are two banks, each holds 32K bytes
        addr           - address in SRAM to write to [0...32767]
        num_bytes      - number of bytes to write from source buffer to SRAM
        value          - byte value to write to SRAM

RETURNS: number of bytes written

}}

  ' test if there is anything to fill?
  if (num_bytes => 1)

    ' select SPI channel for requested SRAM bank
    setLock(bank)

    ' set instruction word ( write byte sequential:8 | address:16 )
    _spi_word := (%00000010 << 16) + (addr)

    ' setup for sequential byte write, this includes the command and address only
    io.SPI_Write_Read( 24, _spi_word, $FF )

    ' now the chip is ready to receive bytes (as many as we want to send)
    repeat _index from 0 to num_bytes-1
'      io.SPI_Write_Read( 8, value, $FF )
      io.SPI_Write_Byte( value )

    ' return bytes written
    clearLock
    return( num_bytes )

  else
    ' catch error
    return 0

' end SRAM_Fill

' /////////////////////////////////////////////////////////////////////////////

PUB SRAM_Read(bank, addr, num_bytes, dest_buffer_ptr) | _index, _data, _spi_word
{{

This function reads a number of bytes from either bank of the SRAM into a buffer.

The 23K256 supports a number of commands, we have only implemented a couple here
in these API functions (single byte read/write, multiple byte read/write).
However, there are many others you can implement, find out more by reviewing
the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

PARMS:  bank            - [0,1] the SRAM bank to read from, there are two banks, each holds 32K bytes
        addr            - address in SRAM to read from [0..32767]
        num_bytes       - number of bytes to read from SRAM into destination buffer
        dest_buffer_ptr - pointer to the buffer of bytes to read SRAM into

RETURNS: number of bytes read

}}

  ' test for anything to read?
  if (num_bytes => 1)

    setLock(bank)

    ' set instruction word ( read byte sequential:8 | address:16 )
    _spi_word := (%00000011 << 16) + (addr)

    ' setup for sequential byte read, this includes the command and address only
    io.SPI_Write_Read( 24, _spi_word, $FF )

    ' FIXME: this is ugly... maybe read mode needs a param for the byte to send on reads to help here)
    '        for now, we'll bytefill zeros and use write/read to get what we want.
    bytefill(dest_buffer_ptr, 0, num_bytes)
    io.SPI_Write_Read_Bytes( dest_buffer_ptr, num_bytes )

'    ' now the chip is ready to be read from
'    repeat _index from 0 to num_bytes-1

      ' read next byte from SRAM into receiving buffer
'      byte [dest_buffer_ptr][_index] := io.SPI_Write_Read( 8, $00, $FF )

    ' return bytes read
    clearLock
    return( num_bytes )
  else
    ' catch error
    return 0

' end SRAM_Read

' /////////////////////////////////////////////////////////////////////////////

PUB SRAM64K_MREAD( addr ) | _spi_word, _data
{{
This function reads a single byte from either bank of the Microchip 23K256
serial SPI SRAMs'. This function is different from the SRAM_Read function
in that it accesses the two banks of SRAM as a contiguous region of 64K bytes
for you, and thus can be though of as a crude "array" access macro, rather
than a function. Ultimately, this should be converted to ASM along with
all the other important peripheral access methods.

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  addr - address in SRAMs to write to [0...65535].

RETURNS: The 8-bit data from the SRAM.

}}

    ' the only tricky thing here is we need to take an address 0..65535 and then
    ' decide which bank it refers to and which byte in that bank, basically, some
    ' bit masking and shifts to accomplish this
    setLock( (addr >> 15) )

    ' set instruction word ( read byte sequential:8 | address:16 | dummy:8 )
    _spi_word := (%00000011 << 24) + (addr << 8) + 0

    ' read the byte of data
    _data := io.SPI_Write_Read( 32, _spi_word, $FF )

    ' return data
    clearLock
    return ( _data )

' end SRAM64K_READ

' /////////////////////////////////////////////////////////////////////////////

PUB SRAM64K_MWRITE( addr, data ) | _spi_word
{{
This function writes a single byte to either bank of the Microchip 23K256
serial SPI SRAMs'. This function is different from the SRAM_Write|Fill functions
in that it accesses the two banks of SRAM as a contiguous region of 64K bytes
for you, and thus can be though of as a crude "array" access macro, rather
than a function. Ultimately, this should be converted to ASM along with
all the other important peripheral access methods.

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  addr - address in SRAMs to write to [0...65535].
        data - the byte to write.

RETURNS: Nothing.

}}

    ' the only tricky thing here is we need to take an address 0..65535 and then
    ' decide which bank it refers to and which byte in that bank, basically, some
    ' bit masking and shifts to accomplish this
    setLock( (addr >> 15) )

    ' set instruction word ( write sequential byte:8 | address:16 | data: 8)
    _spi_word := (%00000010 << 24) + ( (addr & $7FFF) << 8) + data

    ' write the byte of data
    io.SPI_Write_Read( 32, _spi_word, $FF )

    ' de-select SRAM SPI channel
    clearLock

' end SRAM64K_WRITE





PRI setLock(bank)
  ' this version takes a bank # to lock, so as to lock the correct bus
  io.setLock
  io.SPI_Select_Channel(io#SPI_CHAN_SRAM0 + bank)
PRI clearLock
  io.SPI_Select_Channel(io#SPI_CHAN_NULL)
  io.clearLock


DAT

{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
'//////////////////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////


